/*
 * 0. the macro needs a volume mesh
 * 
 * 1. run the macro once. that creates two tags, "splitMe" to be used on planes you want to split and "done". 
 *    the "done" tag is automatically added to planes that has been split already. planes tagged with "done" will not be split again
 * 
 * 2. tag the plane section you want to split with "splitMe". 
 * 
 * 3. if you want surface average reports on these planes, tag the field functions that you want surface average of with the "splitMe"
 * 
 * 4. run the macro again
 */

import java.util.*;

import star.common.*;
import star.base.neo.*;
import star.vis.*;
import star.meshing.*;
import star.base.report.*;
import star.flow.*;

public class planeSplitter_03 extends StarMacro {

  public void execute() {
    execute0();
  }

  String tagName = "splitMe";
  String doneTag = "done";
  String exportPlaneName = "plane";
  Simulation sim;
  String simDir;
  String sep = System.getProperty("file.separator");
  FvRepresentation volumeMeshRep;
  Units lengthUnits;
  Tag done;

  boolean massflowreport = true;
  boolean multivalueMonitor = true;
  String reportGroupStr = "reportGroup";
  String monitorGroupStr = "monitorGroup";
  String planeGroupStr = "planeGroup";

  private void execute0() {

    sim = getActiveSimulation();
    simDir = sim.getSessionDir();

    if (createTag()) {
      return;
    }
    createGroups();

    String releaseString = sim.getReleaseNumber();

    String[] releaseParts = releaseString.split("\\.");
    int majorVersion = Integer.parseInt(releaseParts[0]);
    int minorVersion = Integer.parseInt(releaseParts[1]);
    // multivalued monitors was released in 18.04
    if (majorVersion < 18 || (majorVersion == 18 && minorVersion < 4)) {
      multivalueMonitor = false;
    }
    // parseInt(releaseParts[0]);

    done = sim.get(TagManager.class).getObject(doneTag);

    volumeMeshRep = ((FvRepresentation) sim.getRepresentationManager().getObject("Volume Mesh"));

    lengthUnits = sim.getUnitsManager().getPreferredUnits(Dimensions.Builder().length(1).build());

    Collection<ImplicitPart> planes = getTaggedPlaneSections();
    Collection<FieldFunction> taggedFF = getTaggedFF();

    for (ImplicitPart ps : planes) {
      int iterator = 0;
      MeshPart planeAsMeshpart = plane2part(ps);
      String meshPartName = planeAsMeshpart.getPresentationName();
      Collection<PartSurface> allPartSurfs = planeAsMeshpart.getPartSurfaces();
      Collection<ArbitrarySection> createdArbSections = new ArrayList<>();
      for (PartSurface partSurf : allPartSurfs) { // create an arbitrary section for each part surface
        Collection<NamedObject> inputNamedObjects = ps.getInputPartsCollection();
        ArbitrarySection arbitrarySection_0 = (ArbitrarySection) sim.getPartManager().createArbitraryImplicitPart(
            new NeoObjectVector(new Object[] {}), new NeoObjectVector(new Object[] { partSurf }), resolvePath(""),
            lengthUnits, true);

        arbitrarySection_0.getInputParts().setObjects(inputNamedObjects);

        arbitrarySection_0.setPresentationName(meshPartName + "_" + iterator);

        createdArbSections.add(arbitrarySection_0);

        // move it to the group
        ((ClientServerObjectGroup) sim.getPartManager().getGroupsManager().getObject(planeGroupStr)).getGroupsManager()
            .groupObjects(planeGroupStr, new NeoObjectVector(new Object[] { arbitrarySection_0 }), true);

        iterator = iterator + 1;
      }
      if (!multivalueMonitor) {
        for (ArbitrarySection as : createdArbSections) {
          createReports(taggedFF, as);
        }
      } else {
        createMultiReports(taggedFF, createdArbSections, meshPartName);
      }

      sim.get(TagManager.class).addTags(ps, new NeoObjectVector(new Object[] { done }));
      // setTags(planeSection_0, new NeoObjectVector(new Object[] {userTag_1,
      // userTag_0}));

    }
    sim.println("finished splitting planes and creating reports.");

  }

  private ArbitrarySection createArbitrarySection(PartSurface partSurf) {

    ArbitrarySection arbitrarySection_0 = (ArbitrarySection) sim.getPartManager().createArbitraryImplicitPart(
        new NeoObjectVector(new Object[] { null }), new NeoObjectVector(new Object[] { partSurf }), resolvePath(""),
        lengthUnits, true);

    arbitrarySection_0.setPresentationName("section_1");

    return arbitrarySection_0;

  }

  // create tags
  private boolean createTag() {

    sim.println("tag your plane sections and field functions with the '" + tagName + "' tag");
    sim.println("once a plane has been splitted, it's tagged with the '" + doneTag + "' tag");
    try {
      sim.get(TagManager.class).getObject(doneTag);
    } catch (Exception e) {
      UserTag userTag_0 = sim.get(TagManager.class).createNewUserTag("Tag");
      userTag_0.setPresentationName(doneTag);
      sim.println("created the tag " + doneTag);
    }
    try {
      sim.get(TagManager.class).getObject(tagName);
      return false;
    } catch (Exception e) {
      UserTag userTag_0 = sim.get(TagManager.class).createNewUserTag("Tag");

      userTag_0.setPresentationName(tagName);
      sim.println("created the tag " + tagName);
    }
    return true;
  }

  // find the tagged field functions
  private Collection<FieldFunction> getTaggedFF() {
    Collection<FieldFunction> allFF = sim.getFieldFunctionManager().getObjects();
    Collection<FieldFunction> taggedff = new ArrayList<>();
    for (FieldFunction ff : allFF) {
      if (ff.getTagsInput().toString().contains(tagName)) {
        if (ff.getType().toString().equals("Vector")) {
          VectorMagnitudeFieldFunction vectorMagnitudeFieldFunction_0 = ((VectorMagnitudeFieldFunction) ff
              .getMagnitudeFunction());
          taggedff.add(vectorMagnitudeFieldFunction_0);
        } else {
          taggedff.add(ff);
        }
      }
    }
    return taggedff;
  }

  // export and import the planes as stl
  private MeshPart plane2part(ImplicitPart ps) {

    String planeName = ps.getPresentationName().replaceAll("\\s", "_"); // remove white space " " in name, replace with
                                                                        // underline "_"
    ps.exportDataSourceSTL(volumeMeshRep, false,
        resolvePath(simDir + sep + planeName + ".stl"));

    PartImportManager partImportManager_0 = sim.get(PartImportManager.class);

    partImportManager_0.importStlPart(
        resolvePath(simDir + sep + planeName + ".stl"),
        "OneSurfacePerPatch", lengthUnits, true, 1.0E-5, false);

    MeshPart meshPart_0 = ((MeshPart) sim.get(SimulationPartManager.class).getPart(planeName));

    Collection<PartSurface> allPS = meshPart_0.getPartSurfaceManager().getObjects();
    for (PartSurface partsurf : allPS) {
      meshPart_0.getPartSurfaceManager()
          .splitNonContiguousPartSurfaces(new NeoObjectVector(new Object[] { partsurf }));
    }

    return meshPart_0;
  }

  // find all the tagged planes
  private Collection<ImplicitPart> getTaggedPlaneSections() {

    Collection<Part> allParts = sim.getPartManager().getObjects();
    Collection<ImplicitPart> taggedPlanes = new ArrayList<>();
    for (Part p : allParts) {
      if (p.getTagsInput().toString().contains(tagName) && !p.getTagsInput().toString().contains(doneTag)) {
        if (p instanceof star.vis.ImplicitPart) {
          ImplicitPart ps = (ImplicitPart) p;
          taggedPlanes.add(ps);
        }
      }
    }
    if (taggedPlanes.size() == 0) {
      sim.println("there are no plane sections to be split. tag plane sections you want split with '" + tagName
          + "'. note that plane sections tagged with '" + doneTag + "' will not be split");
    }
    return taggedPlanes;
  }

  // creates the reports
  private void createReports(Collection<FieldFunction> allFF, ArbitrarySection plane) {

    if (massflowreport) {

      MassFlowReport massFlowReport_0 = sim.getReportManager().createReport(MassFlowReport.class);

      massFlowReport_0.getParts().setQuery(null);

      massFlowReport_0.getParts().setObjects(plane);

      massFlowReport_0.setPresentationName("MassFlow_" + plane.getPresentationName());

      ReportMonitor reportMonitor_0 = massFlowReport_0.createMonitor();

      // move part, might want to do this together as a list instead

      ((ClientServerObjectGroup) sim.getReportManager().getGroupsManager().getObject(reportGroupStr))
          .getGroupsManager()
          .groupObjects(reportGroupStr, new NeoObjectVector(new Object[] { massFlowReport_0 }), true);

      ((ClientServerObjectGroup) sim.getMonitorManager().getGroupsManager().getObject(monitorGroupStr))
          .getGroupsManager()
          .groupObjects(monitorGroupStr, new NeoObjectVector(new Object[] { reportMonitor_0 }), true);
    }

    for (FieldFunction ff : allFF) {
      AreaAverageReport areaAverageReport_1 = sim.getReportManager().createReport(AreaAverageReport.class);

      areaAverageReport_1.setFieldFunction(ff);

      areaAverageReport_1.getParts().setQuery(null);

      areaAverageReport_1.getParts().setObjects(plane);

      areaAverageReport_1.setPresentationName(ff.getPresentationName() + "_" + plane.getPresentationName());

      ReportMonitor reportMonitor_0 = areaAverageReport_1.createMonitor();

      // move part, might want to do this together as a list instead

      ((ClientServerObjectGroup) sim.getReportManager().getGroupsManager().getObject(reportGroupStr))
          .getGroupsManager()
          .groupObjects(reportGroupStr, new NeoObjectVector(new Object[] { areaAverageReport_1 }), true);

      ((ClientServerObjectGroup) sim.getMonitorManager().getGroupsManager().getObject(monitorGroupStr))
          .getGroupsManager()
          .groupObjects(monitorGroupStr, new NeoObjectVector(new Object[] { reportMonitor_0 }), true);
    }
  }

  private void createMultiReports(Collection<FieldFunction> allFF, Collection<ArbitrarySection> plane, String name) {

    if (massflowreport) {

      MassFlowReport massFlowReport_0 = sim.getReportManager().createReport(MassFlowReport.class);

      massFlowReport_0.getParts().setQuery(null);

      massFlowReport_0.getParts().setObjects(plane);

      massFlowReport_0.setPresentationName("MassFlow_" + name);

      ReportMonitor reportMonitor_0 = massFlowReport_0.createMonitor();

      reportMonitor_0.getValueType().setSelected(ReportMonitorValueType.Type.PartValue);

      // move part, might want to do this together as a list instead

      ((ClientServerObjectGroup) sim.getReportManager().getGroupsManager().getObject(reportGroupStr))
          .getGroupsManager()
          .groupObjects(reportGroupStr, new NeoObjectVector(new Object[] { massFlowReport_0 }), true);

      ((ClientServerObjectGroup) sim.getMonitorManager().getGroupsManager().getObject(monitorGroupStr))
          .getGroupsManager()
          .groupObjects(monitorGroupStr, new NeoObjectVector(new Object[] { reportMonitor_0 }), true);
    }

    for (FieldFunction ff : allFF) {
      AreaAverageReport areaAverageReport_1 = sim.getReportManager().createReport(AreaAverageReport.class);

      areaAverageReport_1.setFieldFunction(ff);

      areaAverageReport_1.getParts().setQuery(null);

      areaAverageReport_1.getParts().setObjects(plane);

      areaAverageReport_1.setPresentationName(ff.getPresentationName() + "_" + name);

      ReportMonitor reportMonitor_0 = areaAverageReport_1.createMonitor();

      reportMonitor_0.getValueType().setSelected(ReportMonitorValueType.Type.PartValue);

      // move part, might want to do this together as a list instead

      ((ClientServerObjectGroup) sim.getReportManager().getGroupsManager().getObject(reportGroupStr))
          .getGroupsManager()
          .groupObjects(reportGroupStr, new NeoObjectVector(new Object[] { areaAverageReport_1 }), true);

      ((ClientServerObjectGroup) sim.getMonitorManager().getGroupsManager().getObject(monitorGroupStr))
          .getGroupsManager()
          .groupObjects(monitorGroupStr, new NeoObjectVector(new Object[] { reportMonitor_0 }), true);
    }
  }

  // creates the needed groups
  private void createGroups() {

    try {
      sim.getReportManager().getGroupsManager().getObject(reportGroupStr);
    } catch (Exception e) {
      sim.getReportManager().getGroupsManager().createGroup("New Group");

      ((ClientServerObjectGroup) sim.getReportManager().getGroupsManager().getObject("New Group"))
          .setPresentationName(reportGroupStr);
    }
    try {
      sim.getMonitorManager().getGroupsManager().getObject(monitorGroupStr);
    } catch (Exception e) {
      sim.getMonitorManager().getGroupsManager().createGroup("New Group");

      ((ClientServerObjectGroup) sim.getMonitorManager().getGroupsManager().getObject("New Group"))
          .setPresentationName(monitorGroupStr);
    }
    try {
      sim.getPartManager().getGroupsManager().getObject(planeGroupStr);
    } catch (Exception e) {
      sim.getPartManager().getGroupsManager().createGroup("New Group");

      ((ClientServerObjectGroup) sim.getPartManager().getGroupsManager().getObject("New Group"))
          .setPresentationName(planeGroupStr);
    }
  }
}
